@@ -1,9 +1,9 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: evdev.c 463 2008-05-18 10:05:06Z jblache $
+ * $Id: evdev.c 500 2008-07-13 17:53:17Z jblache $
  *
- * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 #include "pommed.h"
 #include "conffile.h"
 #include "evdev.h"
+#include "evloop.h"
 #include "kbd_backlight.h"
 #include "lcd_backlight.h"
 #include "cd_eject.h"
@@ -66,85 +67,31 @@
 #endif
 
 
-/* Open event devices */
-static int ev_fds[EVDEV_MAX];
-
-/* epoll fd */
-static int epfd;
-
-/* inotify fd */
-static int ifd;
-
-
 static int
 evdev_try_add(int fd);
 
 
-static int
-evdev_add(int fd)
-{
-  int i;
-  int ret;
-
-  struct epoll_event epoll_ev;
-
-  for (i = 0; i < EVDEV_MAX; i++)
-    {
-      if (ev_fds[i] == -1)
-	{
-	  ev_fds[i] = fd;
-
-	  break;
-	}
-    }
-
-  epoll_ev.events = EPOLLIN;
-  epoll_ev.data.fd = fd;
-
-  ret = epoll_ctl(epfd, EPOLL_CTL_ADD, fd, &epoll_ev);
-
-  if (ret < 0)
-    {
-      logmsg(LOG_ERR, "Could not add device to epoll: %s", strerror(errno));
-
-      return -1;
-    }
-
-  return 0;
-}
-
-static void
-evdev_remove(int fd)
-{
-  int i;
-  int ret;
-
-  for (i = 0; i < EVDEV_MAX; i++)
-    {
-      if (ev_fds[i] == fd)
-	{
-	  ev_fds[i] = -1;
-
-	  break;
-	}
-    }
-
-  ret = epoll_ctl(epfd, EPOLL_CTL_DEL, fd, NULL);
-
-  if (ret < 0)
-    logmsg(LOG_ERR, "Could not remove device from epoll: %s", strerror(errno));
-
-  close(fd);
-}
-
-
-static void
-evdev_process_events(int fd)
+void
+evdev_process_events(int fd, uint32_t events)
 {
   int ret;
 
   struct input_event ev;
 
+  /* some of the event devices cease to exist when suspending */
+  if (events & (EPOLLERR | EPOLLHUP))
+    {
+      logmsg(LOG_INFO, "Error condition signaled on event device");
+
+      ret = evloop_remove(fd);
+      if (ret < 0)
+	logmsg(LOG_ERR, "Could not remove device from event loop");
+
+      close(fd);
+
+      return;
+    }
+
   ret = read(fd, &ev, sizeof(struct input_event));
 
   if (ret != sizeof(struct input_event))
@@ -162,43 +109,43 @@ evdev_process_events(int fd)
 
       switch (ev.code)
 	{
-	  case K_LCD_BCK_DOWN:
+	  case KEY_BRIGHTNESSDOWN:
 	    logdebug("\nKEY: LCD backlight down\n");
 
 	    mops->lcd_backlight_step(STEP_DOWN);
 	    break;
 
-	  case K_LCD_BCK_UP:
+	  case KEY_BRIGHTNESSUP:
 	    logdebug("\nKEY: LCD backlight up\n");
 
 	    mops->lcd_backlight_step(STEP_UP);
 	    break;
 
-	  case K_AUDIO_MUTE:
+	  case KEY_MUTE:
 	    logdebug("\nKEY: audio mute\n");
 
 	    audio_toggle_mute();
 	    break;
 
-	  case K_AUDIO_DOWN:
+	  case KEY_VOLUMEDOWN:
 	    logdebug("\nKEY: audio down\n");
 
 	    audio_step(STEP_DOWN);
 	    break;
 
-	  case K_AUDIO_UP:
+	  case KEY_VOLUMEUP:
 	    logdebug("\nKEY: audio up\n");
 
 	    audio_step(STEP_UP);
 	    break;
 
-	  case K_VIDEO_TOGGLE:
+	  case KEY_SWITCHVIDEOMODE:
 	    logdebug("\nKEY: video toggle\n");
 
 	    video_switch();
 	    break;
 
-	  case K_KBD_BCK_OFF:
+	  case KEY_KBDILLUMTOGGLE:
 	    logdebug("\nKEY: keyboard backlight off\n");
 
 	    if (!has_kbd_backlight())
@@ -210,7 +157,7 @@ evdev_process_events(int fd)
 	      kbd_backlight_toggle();
 	    break;
 
-	  case K_KBD_BCK_DOWN:
+	  case KEY_KBDILLUMDOWN:
 	    logdebug("\nKEY: keyboard backlight down\n");
 
 	    if (!has_kbd_backlight())
@@ -219,7 +166,7 @@ evdev_process_events(int fd)
 	    kbd_backlight_step(STEP_DOWN);
 	    break;
 
-	  case K_KBD_BCK_UP:
+	  case KEY_KBDILLUMUP:
 	    logdebug("\nKEY: keyboard backlight up\n");
 
 	    if (!has_kbd_backlight())
@@ -228,25 +175,25 @@ evdev_process_events(int fd)
 	    kbd_backlight_step(STEP_UP);
 	    break;
 
-	  case K_CD_EJECT:
+	  case KEY_EJECTCD:
 	    logdebug("\nKEY: CD eject\n");
 
 	    cd_eject();
 	    break;
 
-	  case K_IR_FFWD:
+	  case KEY_NEXTSONG:
 	    logdebug("\nKEY: IR fast forward\n");
 	    break;
 
-	  case K_IR_REWD:
+	  case KEY_PREVIOUSSONG:
 	    logdebug("\nKEY: IR rewind\n");
 	    break;
 
-	  case K_IR_PLAY:
+	  case KEY_PLAYPAUSE:
 	    logdebug("\nKEY: IR play/pause\n");
 	    break;
 
-	  case K_IR_MENU:
+	  case KEY_MENU:
 	    logdebug("\nKEY: IR menu\n");
 	    break;
 
@@ -257,16 +204,6 @@ evdev_process_events(int fd)
 	    break;
 	}
     }
-  else if (ev.type == EV_SND)
-    {
-      /* Beeper device */
-      if ((ev.code == SND_TONE) && (ev.value > 0))
-	{
-	  logdebug("\nBEEP: BEEP!\n");
-
-	  beep_beep(); /* Catch that, Coyote */
-	}
-    }
   else if (ev.type == EV_SW)
     {
       /* Lid switch */
@@ -289,19 +226,32 @@ evdev_process_events(int fd)
 }
 
 
-static void
-evdev_inotify_process(void)
+void
+evdev_inotify_process(int fd, uint32_t events)
 {
   int ret;
-  int fd;
+  int efd;
   int qsize;
 
   struct inotify_event *all_ie;
   struct inotify_event *ie;
   char evdev[32];
 
+  if (events & (EPOLLERR | EPOLLHUP))
+    {
+      logmsg(LOG_WARNING, "inotify fd lost; this should not happen");
+
+      ret = evloop_remove(fd);
+      if (ret < 0)
+	logmsg(LOG_ERR, "Could not remove inotify from event loop");
+
+      close(fd);
+
+      return;
+    }
+
   /* Determine the size of the inotify queue */
-  ret = ioctl(ifd, FIONREAD, &qsize);
+  ret = ioctl(fd, FIONREAD, &qsize);
   if (ret < 0)
     {
       logmsg(LOG_ERR, "Could not determine inotify queue size: %s", strerror(errno));
@@ -317,7 +267,7 @@ evdev_inotify_process(void)
       return;
     }
 
-  ret = read(ifd, all_ie, qsize);
+  ret = read(fd, all_ie, qsize);
   if (ret < 0)
     {
       logmsg(LOG_WARNING, "inotify read failed: %s", strerror(errno));
@@ -358,8 +308,8 @@ evdev_inotify_process(void)
       if ((ret <= 0) || (ret >= sizeof(evdev)))
 	continue;
 
-      fd = open(evdev, O_RDWR);
-      if (fd < 0)
+      efd = open(evdev, O_RDWR);
+      if (efd < 0)
 	{
 	  if (errno != ENOENT)
 	    logmsg(LOG_WARNING, "Could not open %s: %s", evdev, strerror(errno));
@@ -367,75 +317,13 @@ evdev_inotify_process(void)
 	  continue;
 	}
 
-      evdev_try_add(fd);
+      evdev_try_add(efd);
     }
 
   free(all_ie);
 }
 
 
-int
-evdev_event_loop(void)
-{
-  int i;
-  int nfds;
-
-  int inotify = 0;
-
-  struct epoll_event events[MAX_EPOLL_EVENTS];
-
-  nfds = epoll_wait(epfd, events, MAX_EPOLL_EVENTS, LOOP_TIMEOUT);
-
-  if (nfds < 0)
-    {
-      if (errno == EINTR)
-	return 1; /* pommed.c will go on with the events management */
-      else
-	{
-	  logmsg(LOG_ERR, "epoll_wait() error: %s", strerror(errno));
-
-	  return -1; /* pommed.c will exit */
-	}
-    }
-
-
-  for (i = 0; i < nfds; i++)
-    {
-      /* some of the event devices cease to exist when suspending */
-      if (events[i].events & (EPOLLERR | EPOLLHUP))
-	{
-	  logmsg(LOG_INFO, "Error condition signaled on event device");
-
-	  if (events[i].data.fd == beep_info.fd)
-	    logmsg(LOG_WARNING, "Beeper device lost; this should not happen");
-
-	  if (events[i].data.fd == ifd)
-	    {
-	      logmsg(LOG_WARNING, "inotify fd lost; this should not happen");
-	      ifd = -1;
-	    }
-
-	  evdev_remove(events[i].data.fd);
-
-	  continue;
-	}
-
-      if (events[i].events & EPOLLIN)
-	{
-	  if (events[i].data.fd == ifd)
-	    inotify = 1; /* defer inotify events processing */
-	  else
-	    evdev_process_events(events[i].data.fd);
-	}
-    }
-
-  if (inotify)
-    evdev_inotify_process();
-
-  return nfds;
-}
-
-
 #ifdef __powerpc__
 /* PowerBook G4 Titanium */
 static int
@@ -797,7 +685,7 @@ evdev_is_extkbd_alu_wl(unsigned short *id)
 {
   unsigned short product = id[ID_PRODUCT];
 
-  if (id[ID_BUS] != BUS_USB)
+  if (id[ID_BUS] != BUS_BLUETOOTH)
     return 0;
 
   if (id[ID_VENDOR] != USB_VENDOR_ID_APPLE)
@@ -906,9 +794,15 @@ evdev_try_add(int fd)
       return -1;
     }
 
-  ret = evdev_add(fd);
+  ret = evloop_add(fd, EPOLLIN, evdev_process_events);
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Could not add device to event loop");
 
-  return ret;
+      return -1;
+    }
+
+  return 0;
 }
 
 
@@ -916,36 +810,33 @@ static int
 evdev_inotify_init(void)
 {
   int ret;
-  struct epoll_event epoll_ev;
+  int fd;
 
-  ifd = inotify_init();
-  if (ifd < 0)
+  fd = inotify_init();
+  if (fd < 0)
     {
       logmsg(LOG_ERR, "Failed to initialize inotify: %s", strerror(errno));
 
       return -1;
     }
 
-  ret = inotify_add_watch(ifd, EVDEV_DIR, IN_CREATE | IN_ONLYDIR);
+  ret = inotify_add_watch(fd, EVDEV_DIR, IN_CREATE | IN_ONLYDIR);
   if (ret < 0)
     {
       logmsg(LOG_ERR, "Failed to add inotify watch for %s: %s", EVDEV_DIR, strerror(errno));
 
-      close(ifd);
-      ifd = -1;
+      close(fd);
+      fd = -1;
 
       return -1;
     }
 
-  epoll_ev.events = EPOLLIN;
-  epoll_ev.data.fd = ifd;
-
-  ret = epoll_ctl(epfd, EPOLL_CTL_ADD, ifd, &epoll_ev);
+  ret = evloop_add(fd, EPOLLIN, evdev_inotify_process);
   if (ret < 0)
     {
-      logmsg(LOG_ERR, "Failed to add inotify fd to epoll: %s", strerror(errno));
+      logmsg(LOG_ERR, "Failed to add inotify fd to event loop");
 
-      close(ifd);
+      close(fd);
 
       return -1;
     }
@@ -965,19 +856,6 @@ evdev_init(void)
   int ndevs;
   int fd;
 
-  epfd = epoll_create(MAX_EPOLL_EVENTS);
-  if (epfd < 0)
-    {
-      logmsg(LOG_ERR, "Could not create epoll fd: %s", strerror(errno));
-
-      return -1;
-    }
-
-  for (i = 0; i < EVDEV_MAX; i++)
-    {
-      ev_fds[i] = -1;
-    }
-
   ndevs = 0;
   for (i = 0; i < EVDEV_MAX; i++)
     {
@@ -1001,16 +879,6 @@ evdev_init(void)
 
   logdebug("\nFound %d devices\n", ndevs);
 
-  /* Add the beeper device */
-  ret = beep_open_device();
-  if (ret == 0)
-    {
-      ret = evdev_add(beep_info.fd);
-
-      if (ret == 0)
-	  ndevs++;
-    }
-
   /* Initialize inotify */
   evdev_inotify_init();
 
@@ -1020,17 +888,5 @@ evdev_init(void)
 void
 evdev_cleanup(void)
 {
-  int i;
-
-  close(epfd);
-
-  close(ifd);
-
-  for (i = 0; i < EVDEV_MAX; i++)
-    {
-      if (ev_fds[i] == beep_info.fd)
-	beep_close_device();
-      else
-	close(ev_fds[i]);
-    }
+  /* evloop_cleanup() takes care of closing the devices */
 }
