@@ -1,5 +1,5 @@
 /*
- * $Id: kbd_backlight.h 452 2008-04-05 16:07:48Z jblache $
+ * $Id: kbd_backlight.h 482 2008-06-03 17:19:03Z jblache $
  */
 
 #ifndef __KBD_BACKLIGHT_H__
@@ -25,10 +25,11 @@
 
 #define KBD_MASK_AUTO           (KBD_INHIBIT_LID | KBD_INHIBIT_IDLE)
 
-
 #define KBD_USER     0
 #define KBD_AUTO     1
 
+#define KBD_TIMEOUT 200
+
 
 struct _kbd_bck_info
 {
@@ -83,6 +84,9 @@ kbd_backlight_step(int dir);
 void
 kbd_backlight_init(void);
 
+void
+kbd_backlight_cleanup(void);
+
 void
 kbd_backlight_fix_config(void);
 
