@@ -1,9 +1,9 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: kbd_backlight.c 452 2008-04-05 16:07:48Z jblache $
+ * $Id: kbd_backlight.c 482 2008-06-03 17:19:03Z jblache $
  *
- * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
+ * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdint.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -33,6 +34,7 @@
 #include <errno.h>
 
 #include "../pommed.h"
+#include "../evloop.h"
 #include "../conffile.h"
 #include "../kbd_backlight.h"
 #include "../ambient.h"
@@ -255,6 +257,15 @@ kbd_backlight_init(void)
   kbd_bck_info.max = KBD_BACKLIGHT_MAX;
 
   ambient_init(&kbd_bck_info.r_sens, &kbd_bck_info.l_sens);
+
+  kbd_auto_init();
+}
+
+void
+kbd_backlight_cleanup(void)
+{
+  if (has_kbd_backlight())
+    kbd_auto_cleanup();
 }
 
 
@@ -271,5 +282,5 @@ kbd_backlight_fix_config(void)
     kbd_cfg.step = KBD_BACKLIGHT_MAX / 2;
 
   if (kbd_cfg.idle > 0)
-    kbd_cfg.idle = (kbd_cfg.idle * 1000) / LOOP_TIMEOUT;
+    kbd_cfg.idle = (kbd_cfg.idle * 1000) / KBD_TIMEOUT;
 }
