@@ -1,7 +1,7 @@
 /*
  * pommed - Apple laptops hotkeys handler daemon
  *
- * $Id: pommed.c 460 2008-05-18 09:56:51Z jblache $
+ * $Id: pommed.c 494 2008-06-12 19:18:53Z jblache $
  *
  * Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>
  *
@@ -22,11 +22,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdint.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/time.h>
-#include <time.h>
 #include <string.h>
 #include <signal.h>
 
@@ -36,8 +36,7 @@
 #include <errno.h>
 
 #ifndef __powerpc__
-# include <smbios/SystemInfo.h>
-# define check_machine() check_machine_smbios()
+# define check_machine() check_machine_dmi()
 #else
 # define check_machine() check_machine_pmu()
 #endif /* __powerpc__ */
@@ -49,6 +48,7 @@
 #include "lcd_backlight.h"
 #include "cd_eject.h"
 #include "evdev.h"
+#include "evloop.h"
 #include "conffile.h"
 #include "audio.h"
 #include "dbus.h"
@@ -327,14 +327,11 @@ int debug = 0;
 int console = 0;
 
 
-/* Used by signal handlers */
-static int running;
-
-
 void
 logmsg(int level, char *fmt, ...)
 {
   va_list ap;
+  FILE *where = stdout;
 
   va_start(ap, fmt);
 
@@ -343,22 +340,23 @@ logmsg(int level, char *fmt, ...)
       switch (level)
 	{
 	  case LOG_INFO:
-	    fprintf(stdout, "I: ");
+	    fprintf(where, "I: ");
 	    break;
 
 	  case LOG_WARNING:
-	    fprintf(stdout, "W: ");
+	    fprintf(where, "W: ");
 	    break;
 
 	  case LOG_ERR:
-	    fprintf(stdout, "E: ");
+	    where = stderr;
+	    fprintf(where, "E: ");
 	    break;
 
 	  default:
 	    break;
 	}
-      vfprintf(stdout, fmt, ap);
-      fprintf(stdout, "\n");
+      vfprintf(where, fmt, ap);
+      fprintf(where, "\n");
     }
   else
     {
@@ -545,84 +543,122 @@ check_machine_pmu(void)
 #else
 
 static machine_type
-check_machine_smbios(void)
+check_machine_dmi(void)
 {
-  int ret = MACHINE_UNKNOWN;
+  int ret;
 
-  const char *prop;
+  int fd;
+  char buf[32];
+  int i;
 
-  if (geteuid() != 0)
+  char *vendor_node[] =
     {
-      logmsg(LOG_ERR, "root privileges needed for SMBIOS machine detection");
-
-      return MACHINE_ERROR;
-    }
+      "/sys/class/dmi/id/sys_vendor",
+      "/sys/class/dmi/id/board_vendor",
+      "/sys/class/dmi/id/chassis_vendor",
+      "/sys/class/dmi/id/bios_vendor"
+    };
 
   /* Check vendor name */
-  prop = SMBIOSGetVendorName();
-
-  if (prop == NULL)
+  for (i = 0; i < sizeof(vendor_node) / sizeof(vendor_node[0]); i++)
     {
-      logmsg(LOG_ERR, "Could not get vendor name from SMBIOS; running EFI ?");
+      fd = open(vendor_node[i], O_RDONLY);
+      if (fd > 0)
+	break;
+
+      logmsg(LOG_INFO, "Could not open %s: %s", vendor_node[i], strerror(errno));
+    }
+
+  if (fd < 0)
+    {
+      logmsg(LOG_ERR, "Could not determine vendor name");
+
       return MACHINE_ERROR;
     }
 
-  logdebug("SMBIOS vendor name: [%s]\n", prop);
+  memset(buf, 0, sizeof(buf));
 
-  if ((strcmp(prop, "Apple Computer, Inc.") == 0)
-      || (strcmp(prop, "Apple Inc.") == 0))
-    ret = MACHINE_MAC_UNKNOWN;
-
-  SMBIOSFreeMemory(prop);
-
-  if (ret != MACHINE_MAC_UNKNOWN)
-    return ret;
-
-  /* Check system name */
-  prop = SMBIOSGetSystemName();
-
-  if (prop == NULL)
+  ret = read(fd, buf, sizeof(buf) - 1);
+  if (ret < 0)
     {
-      logmsg(LOG_ERR, "Could not get system name from SMBIOS");
+      logmsg(LOG_ERR, "Could not read from %s: %s", vendor_node[i], strerror(errno));
+
+      close(fd);
+      return MACHINE_ERROR;
+    }
+
+  close(fd);
+
+  if (buf[ret - 1] == '\n')
+    buf[ret - 1] = '\0';
+
+  logdebug("DMI vendor name: [%s]\n", buf);
+
+  if ((strcmp(buf, "Apple Computer, Inc.") != 0)
+      && (strcmp(buf, "Apple Inc.") != 0))
+    return MACHINE_UNKNOWN;
+
+  /* Check product name */
+  fd = open("/sys/class/dmi/id/product_name", O_RDONLY);
+  if (fd < 0)
+    {
+      logmsg(LOG_INFO, "Could not open /sys/class/dmi/id/product_name: %s", strerror(errno));
+
       return MACHINE_MAC_UNKNOWN;
     }
 
-  logdebug("SMBIOS system name: [%s]\n", prop);
+  memset(buf, 0, sizeof(buf));
+
+  ret = read(fd, buf, sizeof(buf) - 1);
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Could not read from /sys/class/dmi/id/product_name: %s", strerror(errno));
+
+      close(fd);
+      return MACHINE_MAC_UNKNOWN;
+    }
+
+  close(fd);
+
+  if (buf[ret - 1] == '\n')
+    buf[ret - 1] = '\0';
+
+  logdebug("DMI product name: [%s]\n", buf);
+
+  ret = MACHINE_MAC_UNKNOWN;
 
   /* Core Duo MacBook Pro 15" (January 2006) & 17" (April 2006) */
-  if ((strcmp(prop, "MacBookPro1,1") == 0) || (strcmp(prop, "MacBookPro1,2") == 0))
+  if ((strcmp(buf, "MacBookPro1,1") == 0) || (strcmp(buf, "MacBookPro1,2") == 0))
     ret = MACHINE_MACBOOKPRO_1;
   /* Core2 Duo MacBook Pro 17" & 15" (October 2006) */
-  else if ((strcmp(prop, "MacBookPro2,1") == 0) || (strcmp(prop, "MacBookPro2,2") == 0))
+  else if ((strcmp(buf, "MacBookPro2,1") == 0) || (strcmp(buf, "MacBookPro2,2") == 0))
     ret = MACHINE_MACBOOKPRO_2;
   /* Core2 Duo MacBook Pro 15" & 17" (June 2007) */
-  else if (strcmp(prop, "MacBookPro3,1") == 0)
+  else if (strcmp(buf, "MacBookPro3,1") == 0)
     ret = MACHINE_MACBOOKPRO_3;
   /* Core2 Duo MacBook Pro 15" & 17" (February 2008) */
-  else if (strcmp(prop, "MacBookPro4,1") == 0)
+  else if (strcmp(buf, "MacBookPro4,1") == 0)
     ret = MACHINE_MACBOOKPRO_4;
   /* Core Duo MacBook (May 2006) */
-  else if (strcmp(prop, "MacBook1,1") == 0)
+  else if (strcmp(buf, "MacBook1,1") == 0)
     ret = MACHINE_MACBOOK_1;
   /* Core2 Duo MacBook (November 2006) */
-  else if (strcmp(prop, "MacBook2,1") == 0)
+  else if (strcmp(buf, "MacBook2,1") == 0)
     ret = MACHINE_MACBOOK_2;
   /* Core2 Duo Santa Rosa MacBook (November 2007) */
-  else if (strcmp(prop, "MacBook3,1") == 0)
+  else if (strcmp(buf, "MacBook3,1") == 0)
     ret = MACHINE_MACBOOK_3;
   /* Core2 Duo MacBook (February 2008) */
-  else if (strcmp(prop, "MacBook4,1") == 0)
+  else if (strcmp(buf, "MacBook4,1") == 0)
     ret = MACHINE_MACBOOK_4;
   /* MacBook Air (January 2008) */
-  else if (strcmp(prop, "MacBookAir1,1") == 0)
+  else if (strcmp(buf, "MacBookAir1,1") == 0)
     ret = MACHINE_MACBOOKAIR_1;
   else
-    logmsg(LOG_ERR, "Unknown Apple machine: %s", prop);
+    logmsg(LOG_ERR, "Unknown Apple machine: %s", buf);
 
   if (ret != MACHINE_MAC_UNKNOWN)
-    logmsg(LOG_INFO, "SMBIOS machine check: running on a %s", prop);
-
-  SMBIOSFreeMemory(prop);
+    logmsg(LOG_INFO, "DMI machine check: running on a %s", buf);
 
   return ret;
 }
@@ -632,7 +668,7 @@ check_machine_smbios(void)
 static void
 usage(void)
 {
-  printf("pommed v" M_VERSION " ($Rev: 460 $) Apple laptops hotkeys handler\n");
+  printf("pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler\n");
   printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
 
   printf("Usage:\n");
@@ -646,7 +682,7 @@ usage(void)
 void
 sig_int_term_handler(int signal)
 {
-  running = 0;
+  evloop_stop();
 }
 
 int
@@ -659,10 +695,6 @@ main (int argc, char **argv)
 
   machine_type machine;
 
-  struct timeval tv_now;
-  struct timeval tv_als;
-  struct timeval tv_diff;
-
   while ((c = getopt(argc, argv, "fdv")) != -1)
     {
       switch (c)
@@ -677,7 +709,7 @@ main (int argc, char **argv)
 	    break;
 
 	  case 'v':
-	    printf("pommed v" M_VERSION " ($Rev: 460 $) Apple laptops hotkeys handler\n");
+	    printf("pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler\n");
 	    printf("Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>\n");
 
 	    exit(0);
@@ -686,24 +718,31 @@ main (int argc, char **argv)
 	  default:
 	    usage();
 
-	    exit(-1);
+	    exit(1);
 	    break;
 	}
     }
 
+  if (geteuid() != 0)
+    {
+      logmsg(LOG_ERR, "pommed needs root privileges to operate\n");
+
+      exit(1);
+    }
+
   if (!console)
     {
       openlog("pommed", LOG_PID, LOG_DAEMON);
     }
 
-  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 460 $) Apple laptops hotkeys handler");
+  logmsg(LOG_INFO, "pommed v" M_VERSION " ($Rev: 494 $) Apple laptops hotkeys handler");
   logmsg(LOG_INFO, "Copyright (C) 2006-2008 Julien BLACHE <jb@jblache.org>");
 
   /* Load our configuration */
   ret = config_load();
   if (ret < 0)
     {
-      exit(-1);
+      exit(1);
     }
 
   /* Identify the machine we're running on */
@@ -746,6 +785,13 @@ main (int argc, char **argv)
       exit(1);
     }
 
+  ret = evloop_init();
+  if (ret < 0)
+    {
+      logmsg(LOG_ERR, "Event loop initialization failed");
+      exit (1);
+    }
+
   ret = mops->lcd_backlight_probe();
   if (ret < 0)
     {
@@ -789,7 +835,7 @@ main (int argc, char **argv)
 
 	  evdev_cleanup();
 
-	  exit(-1);
+	  exit(1);
 	}
     }
 
@@ -800,7 +846,7 @@ main (int argc, char **argv)
 
       evdev_cleanup();
 
-      exit(-1);
+      exit(1);
     }
   fprintf(pidfile, "%d\n", getpid());
   fclose(pidfile);
@@ -808,68 +854,15 @@ main (int argc, char **argv)
   /* Spawn the beep thread */
   beep_init();
 
-  gettimeofday(&tv_als, NULL);
-
-  running = 1;
   signal(SIGINT, sig_int_term_handler);
   signal(SIGTERM, sig_int_term_handler);
 
 
-  while (running)
+  do
     {
-      ret = evdev_event_loop();
-
-      gettimeofday(&tv_now, NULL);
-
-      if (ret < 0) /* error */
-	{
-	  break;
-	}
-      else if (ret != 0)
-	{
-	  /* is it time to chek the ambient light sensors and AC state ? */
-	  tv_diff.tv_sec = tv_now.tv_sec - tv_als.tv_sec;
-	  if (tv_diff.tv_sec < 0)
-	    tv_diff.tv_sec = 0;
-
-	  if (tv_diff.tv_sec == 0)
-	    {
-	      tv_diff.tv_usec = tv_now.tv_usec - tv_als.tv_usec;
-	    }
-	  else
-	    {
-	      tv_diff.tv_sec--;
-	      tv_diff.tv_usec = 1000000 - tv_als.tv_usec + tv_now.tv_usec;
-	      tv_diff.tv_usec += tv_diff.tv_sec * 1000000;
-	    }
-
-	  if (tv_diff.tv_usec >= (1000 * LOOP_TIMEOUT))
-	    {
-	      ret = 0; /* go and check ALS, AC state and idle time */
-	    }
-	}
-
-      if (ret == 0)
-	{
-	  /* time to check ambient light sensors, AC state and idle time */
-	  if (has_kbd_backlight())
-	    {
-	      /* Increment keyboard backlight idle timer */
-	      kbd_bck_info.idle++;
-	      if ((kbd_cfg.idle > 0) && (kbd_bck_info.idle > kbd_cfg.idle))
-		kbd_backlight_inhibit_set(KBD_INHIBIT_IDLE);
-
-	      kbd_backlight_ambient_check();
-	    }
-
-	  power_check_ac_state();
-
-	  tv_als = tv_now;
-	}
-
-      /* Process DBus requests */
-      mbpdbus_process_requests();
+      ret = evloop_iteration();
     }
+  while (ret >= 0);
 
   evdev_cleanup();
 
@@ -877,6 +870,12 @@ main (int argc, char **argv)
 
   mbpdbus_cleanup();
 
+  kbd_backlight_cleanup();
+
+  power_cleanup();
+
+  evloop_cleanup();
+
   config_cleanup();
 
   logmsg(LOG_INFO, "Exiting");
