@@ -1,5 +1,5 @@
 /*
- * $Id: power.h 340 2007-07-26 15:23:27Z jblache $
+ * $Id: power.h 482 2008-06-03 17:19:03Z jblache $
  */
 
 #ifndef __POWER_H__
@@ -12,14 +12,14 @@
 #define AC_STATE_ONLINE   1
 #define AC_STATE_OFFLINE  0
 
-
-void
-power_check_ac_state(void);
+#define POWER_TIMEOUT 200
 
 
 void
 power_init(void);
 
+void
+power_cleanup(void);
 
 #endif /* !__POWER_H__ */
 
