@@ -1,7 +1,7 @@
 /*
  * wmpomme -- WindowMaker dockapp for use with pommed
  *
- * $Id: wmpomme.c 347 2007-07-27 14:17:52Z jblache $
+ * $Id: wmpomme.c 413 2007-12-04 11:15:51Z jblache $
  *
  * Copyright (C) 2006-2007 Julien BLACHE <jb@jblache.org>
  *
@@ -33,8 +33,12 @@
 #include <unistd.h>
 #include <ctype.h>
 
+#include <sys/types.h>
+#include <sys/wait.h>
+
 #include <signal.h>
 
+#include <X11/Xlib.h>
 #include <X11/xpm.h>
 
 #include <dbus/dbus.h>
@@ -42,7 +46,8 @@
 #include "wmgeneral.h"
 #include "wmpomme-master.xpm"
 
-#include "../dbus-client/dbus-client.h"
+#include "../client-common/dbus-client.h"
+#include "../client-common/video-client.h"
 
 
 struct {
@@ -91,7 +96,7 @@ wmmbp_dbus_init(void)
 
   signals = MBP_DBUS_SIG_LCD | MBP_DBUS_SIG_KBD
             | MBP_DBUS_SIG_VOL | MBP_DBUS_SIG_MUTE
-            | MBP_DBUS_SIG_LIGHT;
+            | MBP_DBUS_SIG_LIGHT | MBP_DBUS_SIG_VIDEO;
 
   conn = mbp_dbus_init(&dbus_err, signals);
 
@@ -176,6 +181,10 @@ mbp_dbus_listen(int timeout)
 				DBUS_TYPE_BOOLEAN, &mbp.snd_mute,
 				DBUS_TYPE_INVALID);
 	}
+      else if (dbus_message_is_signal(msg, "org.pommed.signal.videoSwitch", "videoSwitch"))
+	{
+	  mbp_video_switch(display);
+	}
       else if (dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
 	{
 	  printf("DBus disconnected\n");
@@ -536,6 +545,18 @@ sig_int_term_handler(int signo)
   running = 0;
 }
 
+void
+sig_chld_handler(int signo)
+{
+  int ret;
+
+  do
+    {
+      ret = waitpid(-1, NULL, WNOHANG);
+    }
+  while (ret > 0);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -543,6 +564,7 @@ main(int argc, char **argv)
 
   signal(SIGINT, sig_int_term_handler);
   signal(SIGTERM, sig_int_term_handler);
+  signal(SIGCHLD, sig_chld_handler);
 
   ProgName = argv[0];
   if (strlen(ProgName) >= 5)
